/**
  *  \file t_array.cc
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "cpluslib/array.h"

#include "tests.h"
#include "cpluslib/format.h"

static string_t
fmt(array2<const int> a)
{
    string_t result;
    for (std::size_t y = 0; y < a.size_y(); ++y) {
        for (std::size_t x = 0; x < a.size_x(); ++x) {
            result += format("%5d", a[y][x]);
        }
        result += "\n";
    }
    result += "\n";
    return result;
}

static string_t
fmt(array3<int> a)
{
    string_t result;
    for (std::size_t z = 0; z < a.size_z(); ++z) {
        result += format("z = %d:\n", z);
        result += fmt(a[z]);
    }
    return result;
}

static int
corr(array1<int> a, array1<int> b)
{
    int sum = 0;
    for (std::size_t i = 0; i < a.size_x(); ++i)
        sum += a[i] * b[i];
    return sum;
}

void
TestArray::testArray()
{
    static int a[3][4] = { { 1, 2, 3, 4 }, { 5, 6, 7, 8 }, { 9, 10, 11, 12 } };
    static int b[] = { 55, 66, 77, 88, 99 };
    TS_ASSERT_EQUALS(fmt(array2<int>(a)),
                     "    1    2    3    4\n"
                     "    5    6    7    8\n"
                     "    9   10   11   12\n\n");
    TS_ASSERT_EQUALS(fmt(array2<int>(a).transposed_xy()),
                     "    1    5    9\n"
                     "    2    6   10\n"
                     "    3    7   11\n"
                     "    4    8   12\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_2d()),
                     "   55   66   77   88   99\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).with_stride_x(2).make_2d()),
                     "   55   77   99\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).subrange_x(2,4).make_2d()),
                     "   77   88\n\n");
    TS_ASSERT_EQUALS(fmt(array2<int>(a).make_3d()),
                     "z = 0:\n"
                     "    1    5    9\n"
                     "    2    6   10\n"
                     "    3    7   11\n"
                     "    4    8   12\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_3d()),
                     "z = 0:\n"
                     "   55   66   77   88   99\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_3d().transposed_xz()),
                     "z = 0:\n"
                     "   55\n\n"
                     "z = 1:\n"
                     "   66\n\n"
                     "z = 2:\n"
                     "   77\n\n"
                     "z = 3:\n"
                     "   88\n\n"
                     "z = 4:\n"
                     "   99\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_3d().transposed_yz()),
                     "z = 0:\n"
                     "   55   66   77   88   99\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_3d().transposed_xy()),
                     "z = 0:\n"
                     "   55\n"
                     "   66\n"
                     "   77\n"
                     "   88\n"
                     "   99\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_2d(2)),
                     "   55   66\n"
                     "   77   88\n\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_2d(1)),
                     "   55\n"
                     "   66\n"
                     "   77\n"
                     "   88\n"
                     "   99\n"
                     "\n");
    TS_ASSERT_EQUALS(fmt(array1<int>(b).make_3d(2,2)),
                     "z = 0:\n"
                     "   55   66\n"
                     "   77   88\n\n");

    TS_ASSERT_EQUALS(corr(array1<int>(a[0]), array1<int>(a[1])), 70);
}
